@@ -39,10 +39,11 @@ class TaskRun(RunTaskProcessor):
         callarg = shlex.split(args['prog'])
         
         # is this program allowed?
-        if not ('*' in self.tpconf['safebin'] or callarg[0] in self.tpconf['safebin']):
-            ts.oc.log.error('{} is not in safebin: {}. Maybe try --tpconf {}:safebin={}'.format(callarg[0], str(self.tpconf['safebin']), self.code,callarg[0] ))
-            ts.stop()            
-            return None
+        if ts.oc.tpconf_enabled:
+            if not ('*' in self.tpconf['safebin'] or callarg[0] in self.tpconf['safebin']):
+                ts.stop('{} is not in safebin: {}. Maybe try --tpconf {}:safebin={}'.format(callarg[0], str(self.tpconf['safebin']), self.code,callarg[0] ))
+                #ts.stop()            
+                return None
 
         try:
             user = pwd.getpwnam(self.tpconf['user'])            
