@@ -22,6 +22,8 @@ import okerrclient
 from okerrclient.exceptions import OkerrExc, OkerrNoKey, OkerrNoTextID, OkerrKeyAuth, \
     OkerrBadData, OkerrBadMethod, OkerrClientServer
 
+from okerrclient.utils import dhms
+
 #
 # Taskprocessor is parent class for processor, which implements sequence code
 # like 'DIR' or 'JDUMP'
@@ -177,7 +179,10 @@ class TaskSeq():
     dumpname = None
     lastcmd = None
     
-    def stop(self):
+    def stop(self, details=None):
+        if details:
+            self.oc.error(details)
+            self.details = details
         self._stop = True
 
     # taskseq.setsteps
@@ -399,7 +404,6 @@ class TaskSeq():
    #
    
     def run(self,data=None,steps=None):
-        # print("run ({}) {}".deformat(self.iname,self.route))
 
         def strcmd(r):
             arg=''
@@ -410,22 +414,19 @@ class TaskSeq():
 
 
         for r in self.fullroute:        
+
             if self.steps is not None:                
                 if self.steps == 0:                
                     if self.dumpname is None or self.dumpname == self.iname:
                         for d in self.dump:
                             self.runcmd(d,data,dict())
                         self.stop()
+                        break
                 else:                     
                     self.steps -= 1
                                   
             self.lastcmd = self.route.pop(0)
     
-            if self._stop:
-                self.oc.log.debug('sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
-                data = self.except_data
-                break
-
             r = r.copy()
 
             cmd = r['command']
@@ -461,6 +462,13 @@ class TaskSeq():
             data = self.runcmd(cmd,data,r)
             
             #self.oc.log.debug('output: {}'.format(data))
+
+            if self._stop:
+                self.oc.log.debug('sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
+                data = self.except_data
+                break
+
+        # END OF ROUTE
     
         # print('i: {}, data: {}'.format(self.iname,data))
         
@@ -497,7 +505,14 @@ class TaskAlias(TaskProcessor):
     
     def run(self,ts,data,args):
         for code in self.alias:
-            data = ts.tp[code].run(ts,data,args)
+            cargs = ts.mkargs(code)
+            # cargs = dict(ts.tp[code].defargs)
+            for k,v in args.items():
+                if k in cargs:
+                    cargs[k] = v
+            
+            # print("ALIAS run {} cargs {}".format(code, cargs))
+            data = ts.tp[code].run(ts,data,cargs)
 
         return data
 
@@ -574,6 +589,19 @@ class TaskExcept(GenTaskProcessor):
     
 TaskExcept('EXCEPT',TaskSeq)
 
+class TaskErrors(GenTaskProcessor):
+    help = 'okerrclient errors summary'
+
+    def run(self,ts,data,args):
+        if last_error:
+            ts.details = "{} ({}s ago)".format(ts.oc.last_error, dhms(time.time() - ts.oc.last_error_time))        
+        else:
+            ts.details = ""
+        return ts.oc.errors
+    
+TaskErrors('ERRORS',TaskSeq)
+
+
 
 class TaskSet(GenTaskProcessor):
     
@@ -642,21 +670,22 @@ class TaskVersion(GenTaskProcessor):
     help = 'Ask server, if current okerrclient version is OK or ERR. (ERR = need to update)'
     def run(self,ts,data,args):
         product = "okerrclient"
-        url = 'http://cp.okerr.com/api/check_version/{}/{}'.format(product, okerrclient.ver)
+        url = 'https://cp.okerr.com/api/check_version/{}/{}'.format(product, okerrclient.ver)
         try:
-            r = requests.get(url)
+            r = requests.get(url, timeout=30)
             d = json.loads(r.text)
             ts.details = d['details']
             return d['status']
         except Exception as e:
-            ts.details = "version check error"
+            ts.details = "version check error: {}".format(str(e))
+            ts.oc.error("version check error: {}".format(str(e)))
             return "ERR"
         
 TaskVersion('VERSION', TaskSeq)
 
 
 class TaskStr(GenTaskProcessor):
-    help = 'if str argument is set, returns str. Joins list by \n, otherwise returns str(data).'
+    help = 'if str argument is set, returns str. Joins list by \n, otherwise returns str(data) or empty string.'
  
     store_argline='str'
     parse_args=False
@@ -666,6 +695,9 @@ class TaskStr(GenTaskProcessor):
         if args['str']:
             s = args['str']
         else:
+            if data is None:
+                return ''
+                        
             try:       
                 s = "\n".join(data)
             except:
@@ -993,9 +1025,7 @@ class TaskMkSeq(GenTaskProcessor):
 
 
     # mkseq.run
-    def run(self,ts,data,args,v=None,keypath=None):
-    
-               
+    def run(self,ts,data,args,v=None,keypath=None):                   
             
         if keypath is None:
             keypath = list()
@@ -1013,11 +1043,12 @@ class TaskMkSeq(GenTaskProcessor):
             if args['path']:
                 try:
                     data,path = ts.oc.altkeypath(args['path'])
+                    
                     # everything is fine!
                     self.origkeypath = path.split(':')                
                     self.keypath=list()
                 except OkerrExc as e:
-                    ts.oc.log.error('get keys error: {}'.format(str(e)))
+                    ts.oc.error('get keys error: {}'.format(str(e)))
                     return None                
                     
             else:
@@ -1031,7 +1062,6 @@ class TaskMkSeq(GenTaskProcessor):
         if v is None:
             v=dict()   
                         
-
         if isinstance(data,list):
             ddata={}
             # convert data to dict
@@ -1040,8 +1070,10 @@ class TaskMkSeq(GenTaskProcessor):
                 if ss:
                     ddata[i] = ss
             data = ddata
-                 
-        
+     
+        # clean up this dict from commends
+        data = {k: v for k, v in data.iteritems() if isinstance(v, dict) or not v.startswith('#')}
+                        
         if isinstance(data,dict) and hasdict(data):
             for sdname,subdata in data.items():
                 if isinstance(subdata,str):
@@ -1137,7 +1169,12 @@ class TaskSave(GenTaskProcessor):
     }
 
     def run(self,ts,data,args):               
-        ts.setvar(args['varname'], data)    
+        if not args['varname'] and isinstance(data, dict):
+            for k,v in data.items():
+                ts.setvar(k, v) 
+        else:
+            ts.setvar(args['varname'], data)    
+        
         return data
 
 TaskSave('SAVE', TaskSeq)
